######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# BUGS model to estimate number of fish swimming up to Scorff
# Chapter 5

######################################################################################
######################################################################################


rm(list=ls())
library(boot)
library("BRugs")
library(MASS)

# Load/check model, load data and inits

model <- "model_BetaBinMultiple_Scorff.txt"

# Data to run model for Years 95 only
data <- "data_BetaBinMultiple_Scorff95.txt"
inits1 <- "inits1_BetaBinMultiple_Scorff95.txt"
inits2 <- "inits2_BetaBinMultiple_Scorff95.txt"
inits3 <- "inits3_BetaBinMultiple_Scorff95.txt"

# Data to run model for Years 95 - 99
# data   <- "data_BetaBinMultiple_Scorff95_99.txt"
# inits1 <- "inits1_BetaBinMultiple_Scorff95_99.txt"
# inits2 <- "inits2_BetaBinMultiple_Scorff95_99.txt"
# inits3 <- "inits3_BetaBinMultiple_Scorff95_99.txt"

data.source <- source(data)$value
Y1 <- data.source$Y1


# MCMC iterations

# Number iteration for the "burn in" phase
n.burnIn <- 10000
# Number iteration for inferences
n.iter <- 10000
thin <- 10
# Number of chains
n.Chains <- 3

# ------------------------------------------------------------------------------------
# Run model 
# ------------------------------------------------------------------------------------

modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelInits(c(inits1, inits2, inits3)) 	

# Burn-in period

modelUpdate(n.burnIn, thin = thin) 

# Inferences

samplesSet(c("kappa","theta","alpha","beta","tau","delta","pi","Zmc","Zuc","Zms","Zus")) 
modelUpdate(numUpdates = n.iter, thin = thin) 


# -------------------------------------------------------------------------
# Results
# -------------------------------------------------------------------------


# Have a look on MCMC chains

windows()
Results <- samplesHistory("*", mfrow = c(2, 1))

# Get results

delta <- samplesSample("delta")
alpha <- samplesSample("alpha")
beta <- samplesSample("beta")
theta <- samplesSample("theta")
pi <- samplesSample("pi")
tau <- samplesSample("tau")

kappa <- as.data.frame(as.matrix(buildMCMC("kappa")))
Zmc <- as.data.frame(as.matrix(buildMCMC("Zmc")))
Zuc <- as.data.frame(as.matrix(buildMCMC("Zuc")))
Zms <- as.data.frame(as.matrix(buildMCMC("Zms")))
Zus <- as.data.frame(as.matrix(buildMCMC("Zus")))




# ------------------------------------------------------------------------------------
# Table of results
# ------------------------------------------------------------------------------------

samplesStats("*")

# Correlations

param <- cbind(kappa[1]$kappa,alpha,beta,delta,tau,pi,theta)
round(100*cor(param))



# ------------------------------------------------------------------------------------
# Make graphic representation of results
# ------------------------------------------------------------------------------------

size.text <- 1.3

# Prior / Posterior distribution of kappa 
# Works for model with data 1995
# ------------------------------------------------------

# windows()

res <- 6
name_figure <- "PostRunSize1995.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), oma = c(1,1,1,1))

x <- seq(0,4000,by=10)
prior <- dgamma(x, shape=2.4, scale = 500)
sample <- kappa[1]$kappa

plot(x=0, y=-1, xlim=c(0,4000),ylim=c(0,0.006), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)

mtext(side=1, text = expression(kappa), bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

legend(	legend = c("Prior","Posterior"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 

dev.off()




# Joint distribution of kappa / theta
# Works for model with data 1995
# ------------------------------------------------------

# windows()

res <- 6
name_figure <- "JointPostTetaKappa1995.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

def.par <- par(no.readonly = TRUE) 		# save default, for resetting...

x <- kappa[1]$kappa
y <- theta

indice <- which(x<=quantile(x, prob = 0.995))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100

nb.points = 3000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, 
	xlab="", ylab="", pch=".", cex.lab = 1.5)
dens2d <- kde2d(x=x, y=y, n = 100)
contour(dens2d , nlevels = 5, drawlabels=F, col = "blue", lw = 2, add = T) 
mtext(text=expression(kappa), side=1, line=3, cex=size.text)
mtext(text=expression(theta), side=2, line=3, cex=size.text)

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for  ",kappa)), side=3, line=1, cex=size.text)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for  ",theta)), side=4, line = 1, las = 0, cex=size.text)

par(def.par)

dev.off()




# Prior / Posterior distribution of kappa 
# Works for model with data 1995-1999
# ------------------------------------------------------

# windows()

res <- 5
name_figure <- "PostRunSize5Years.png"
png(filename = name_figure, height = 750*res, width = 1000*res, res=72*res)

par(mfrow = c(2,3), oma = c(3,2,1,1), mar = c(2,2,2,2))

ylabel <- c("Density"," "," ","Density"," ")
xlabel <- c("1995","1996","1997","1998","1999")

for(i in 1:5) {

x <- seq(0,4000,by=1)
prior <- dgamma(x, shape=2.4, scale = 500)
sample <- kappa[i]$kappa

plot(x=0, y=-1, xlim=c(0,1000),ylim=c(0,0.03), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)

mtext(side=1, text = xlabel[i], bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = ylabel[i], bty="n", line=2, cex=size.text, font = 1)

}

dev.off()





# Prior / Posterior for all parameters
# Works for model with data 1995 only and data 1995-1999
# ------------------------------------------------------

# windows()

res <- 5
name_figure <- "PostParam1995.png"
# name_figure <- "PostParam5Years.png"
png(filename = name_figure, height = 750*res, width = 1000*res, res=72*res)

par(mfrow = c(2,3), oma = c(3,2,1,1), mar = c(2,2,2,2))

line.wd = 1

x <- seq(0,1,by=0.001)

theta.p <- dbeta(x,shape1=1.53,shape2=1.53)
alpha.p <- dbeta(x,shape1=10,shape2=1.5)
beta.p <- dbeta(x,shape1=1.3,shape2=2.2)
tau.p <- dbeta(x,shape1=5.5,shape2=1.5)
delta.p <- dbeta(x,shape1=1.53,shape2=1.53)
pi.p <- dbeta(x,shape1=1.6,shape2=11)

prior <- theta.p
sample <- theta
plot(x=0, y=-1, ylim = c(0,20), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = F, cex.axis=1, las=1) 
mtext(side=1, text = expression(theta), bty="n", line=2, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

prior <- alpha.p
sample <- alpha
plot(x=0, y=-1, ylim = c(0,6), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = F, cex.axis=1, las=1) 
mtext(side=1, text = expression(alpha), bty="n", line=2, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
#mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

prior <- beta.p
sample <- beta
plot(x=0, y=-1, ylim = c(0,20), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = F, cex.axis=1, las=1) 
mtext(side=1, text = expression(beta), bty="n", line=2, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
#mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

prior <- tau.p
sample <- tau
plot(x=0, y=-1, ylim = c(0,6), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
mtext(side=1, text = expression(tau), bty="n", line=3, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

prior <- delta.p
sample <- delta
plot(x=0, y=-1, ylim = c(0,16), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
mtext(side=1, text = expression(delta), bty="n", line=3, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
#mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

prior <- pi.p
sample <- pi
plot(x=0, y=-1, ylim = c(0,30), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
points(x,prior,xlab="",ylab="",type="l", bty="n", lty=2)
lines(density(sample), lty=1)
axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
mtext(side=1, text = expression(pi), bty="n", line=3, cex=size.text, font = 1)
axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=1)
#mtext(side = 2, text = "Density", bty="n", line=2, cex=size.text, font = 1)

dev.off()


